#ifndef TOOL_H
#define TOOL_H

#include "../conf.h"
#include "../word-bank/bank.h"
#include <math.h>
#include <stdbool.h>

// Tool maximum possibility set
#define POSSIBILITY_SET 10

typedef struct _possibility_t {
    char *word;
    double entropy;
} possibility_t;

typedef struct _matches_t {
    char **words;
    int count;
} matches_t;

/**
 * @brief Initialize the tool
 */
void init_tool();

/**
 * @brief Destroy the tool
 */
void destroy_tool();

/**
 * @brief Destroy possibilities
 * @param p
 */
void destroy_possibilities(possibility_t **p);

/**
 * @brief Does a string contains a char
 *
 * @param foo
 * @param c
 * @return
 */
bool string_contains(char *foo, char c);

/**
 * @brief Compute all possible matches
 *
 * @param word
 * @param pattern
 * @return
 */
possibility_t **compute_matches(char *word, Pattern *pattern);

/**
 * @brief Save computed matches
 *
 * @param possibilities
 */
void save_computed_matches(possibility_t **matches);

/**
 * @brief Get the remaning bank count
 *
 * @return
 */
int get_remaining_bank_count();

/**
 * @brief Get the remaining bank
 *
 * @return
 */
char **get_remaining_bank();

#endif
