#ifndef UI_H_
#define UI_H_

#include "../conf.h"
#include <ncurses.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Placements
#define TERM_MID_LINES (LINES / 2)
#define TERM_MID_COLS (COLS / 2)

// Color pairs
#define GREEN_PAIR 1
#define ORANGE_PAIR 2
#define WHITE_PAIR 3
#define RED_PAIR 4

// Menu choices
#define MENU_SOLO 0
#define MENU_1_V_1 1
#define MENU_COMPUTER 2
#define MENU_QUIT 3

// Title lengths
#define TITLES_LINE_COUNT 8

// Title IDs
#define MAIN_TITLE_ID 0
#define SOLO_TITLE_ID 1
#define ONE_V_ONE_TITLE_ID 2
#define COMPUTER_TITLE_ID 3

// Control key handler
#define ctrl(x) ((x)&0x1f)

// Game controls
#define KEYBIND_HELP (ctrl('h'))
#define KEYBIND_TAG (ctrl('t')) // Tool Assisted Game
#define KEYBIND_QUIT (ctrl('w'))
#define KEYBIND_GUESS '\n'

/**
 * @brief Print a text at the center of the given line
 *
 * @param foo
 * @param line
 */
void printw_center(char *foo, int line);

/**
 * @brief Print a big title
 *
 * @param id
 */
void draw_title(int id);

/**
 * @brief Show the answer
 *
 * @param answer
 */
void show_answer(char *answer);

/**
 * @brief Hide the answer
 *
 */
void hide_answer();

/**
 * @brief Initialize the terminal
 *
 * @return true
 * @return false
 */
bool init_term();

/**
 * @brief Cleanup the terminal
 *
 */
void cleanup_term();

/**
 * @brief Show the game menu
 *
 * @return int Choice id
 */
int show_menu();

/**
 * @brief Toggle the help window
 *
 */
void toggle_help();

/**
 * @brief Toggle the tool window
 */
void show_tool();

/**
 * @brief Update the tool subwindow
 *
 * @param count
 * @param words
 * @param entropies
 */
void update_tool(int count, char words[count][WORD_LENGHT + 1], double *entropies);

/**
 * @brief Show the gameboard
 *
 * @param tries_count
 * @param word_length
 * @param tries
 * @param patterns
 * @param scores
 * @param mode
 * @param p2_turn
 * @param current_turn
 */
void show_gameboard(int tries_count, int word_length, char **tries, Pattern **patterns, int* scores, Gamemode mode, bool p2_turn, int current_turn);

/**
 * @brief Show the winner
 *
 * @param p2_turn
 * @param scores
 */
void show_winner(bool p2_turn, int* scores);

#endif
