#include "bank.h"

//==========================
//        PRIVATE
//==========================
FILE *bank_file;
char **bank = NULL;
int bank_count = 0;

/**
 * @brief Initialize the bank file
 *
 */
void init_bank() {
    bank_file = fopen(BANK_PATH, "r");

    if (bank_file == NULL) {
        perror("file");
        exit(EXIT_FAILURE);
    }
}

/**
 * @brief Word picking condition
 *
 * @param word
 * @return true
 * @return false
 */
bool pick_condition(char *word) {
    if (strlen(word) != WORD_LENGHT)
        return false;

    // Check for non-alphabet letters
    for (size_t i = 0; i < strlen(word); i++) {
        if (!((word[i] >= 'a' && word[i] <= 'z') || (word[i] >= 'A' && word[i] <= 'Z')))
            return false;
    }

    return true;
}

/**
 * @brief Cleanup the bank entries
 *
 * @param tmp_bank
 * @param valid_word_count
 */
void cleanup_bank(char **tmp_bank, int valid_word_count) {
    bank = malloc(valid_word_count * sizeof(char **));

    int bank_idx = 0;
    for (int i = 0; i < bank_count; i++) {
        if (!pick_condition(tmp_bank[i]))
            continue;

        bank[bank_idx] = malloc((strlen(tmp_bank[i]) + 1) * sizeof(char));
        strcpy(bank[bank_idx], tmp_bank[i]);

        for (size_t j = 0; j < strlen(bank[bank_idx]); j++)
            bank[bank_idx][j] = toupper(bank[bank_idx][j]);

        bank_idx++;
    }

    for (int i = 0; i < bank_count; i++)
        free(tmp_bank[i]);
    free(tmp_bank);

    bank_count = valid_word_count;
}

/**
 * @brief Load the bank content
 *
 */
void load_bank() {
    char *lineBuf = NULL;
    size_t n = 0;
    size_t lineLength = 0;
    int valid_word_count = 0;
    char **tmp = malloc(count_words() * sizeof(char **));

    while ((lineLength = getline(&lineBuf, &n, bank_file)) != (size_t)-1) {
        lineBuf[strcspn(lineBuf, "\n")] = '\0'; // Set the last char of the string to EOL
        tmp[bank_count] = malloc((lineLength + 1) * sizeof(char));
        if (pick_condition(lineBuf))
            valid_word_count++;
        strcpy(tmp[bank_count], lineBuf);
        bank_count++;
    }
    free(lineBuf);
    fclose(bank_file);

    cleanup_bank(tmp, valid_word_count);
}

//==========================
//        PUBLIC
//==========================
int count_words() {
    char ch;
    int count = 0;

    rewind(bank_file);
    do {
        ch = fgetc(bank_file);
        if (ch == '\n')
            count++;
    } while (ch != EOF);
    rewind(bank_file);

    return count;
}

char *get_random_word() {
    if (bank == NULL) {
        init_bank();
        load_bank();
    }

    return bank[rand() % bank_count];
}

char **get_bank() {
    if (bank == NULL) {
        init_bank();
        load_bank();
    }

    return bank;
}

int get_bank_size() {
    if (bank == NULL) {
        init_bank();
        load_bank();
    }

    return bank_count;
}