#ifndef WORDLE_H_
#define WORDLE_H_

#include "../conf.h"
#include "../tool/tool.h"
#include "../ui/ui.h"
#include "../word-bank/bank.h"
#include <ctype.h>
#include <stdbool.h>
#include <string.h>

/**
 * @brief Initialize the game
 */
void initialize_game();

/**
 * @brief Terminate a game
 */
void terminate_game();

/**
 * @brief Destroy a game
 *
 */
void destroy_game();

/**
 * @brief Restart the game
 *
 */
void restart_game();

/**
 * @brief Condition(s) to lose a game
 *
 * @return true
 * @return false
 */
bool lose_condition();

/**
 * @brief Condition(s) to win a game
 *
 * @return true
 * @return false
 */
bool win_condition();

/**
 * @brief Check if the given letter is a lowercase letter only (ASCII code)
 *
 * @param key
 * @return true
 * @return false
 */
bool validate_letter(int key);

/**
 * @brief Control the placement of each letter
 */
Pattern *generate_pattern();

/**
 * @brief Is a try valid
 *
 * @param try
 * @return true
 * @return false
 */
bool validate_guess(char current_try[WORD_LENGHT]);

/**
 * @brief Set the gamemode
 *
 * @param menu_gamemode
 */
void set_gamemode(Gamemode menu_gamemode);

/**
 * @brief Set the answer
 *
 * @warning Should only be used in test procedure or privately
 * @param answer
 */
void set_answer(char answer[WORD_LENGHT]);

/**
 * @brief Set the current try
 *
 * @warning Should only be used in test procedure
 * @param current_try
 */
void set_try(char current_try[WORD_LENGHT]);

/**
 * @brief Get the gamemode
 *
 * @return Gamemode
 */
Gamemode get_gamemode();

/**
 * @brief Get the answer
 *
 * @return
 */
char *get_answer();

/**
 * @brief Get the try
 *
 * @return
 */
char *get_try();

/**
 * @brief Launch the game
 *
 */
void launch_game();

/**
 * @brief Is the game finished
 *
 * @return true
 * @return false
 */
bool game_finished();

#endif